#include "taster.h"

volatile uint8_t taster;

void get_taster (taste_t * const ptast, uint8_t tast)
{
    // Nichts tun, falls die Anwendung den Wert aus `taster' noch nicht
    // abgehohlt hat 
    if (taster)
        return;
        
    // ist Taster gedrckt? (low aktiv)
    tast = ! tast;
    
    // alten Taster-Zustand lesen
    uint8_t taster_old = ptast->old;
    uint8_t mode  = ptast->mode;
    
#ifndef TASTER_SAME_MODE    
    // skip?
    if (KM_SKIP == mode)
        tast = taster_old;
#endif
    
    // Taster-Zustand merken
    ptast->old = tast;

    // Was wurde gedrckt/losgelassen...?
    uint8_t pressed =  taster_old &  tast;
    uint8_t release =  taster_old & ~tast;
    uint8_t press   = ~taster_old &  tast;

    tast = 0;

    uint8_t delay = ptast->delay;
    uint8_t key   = ptast->key;

#ifdef TASTER_SAME_MODE    
    // Let GCC throw away superflous code if there is some extra 
    // knowledge. Even though there is more C code in that case
    // the code size will reduce.
    mode = TASTER_SAME_MODE;
#endif // TASTER_SAME_MODE    

    // Taste wurde eben gedrckt
    if (press)
    {
        if (mode != KM_LONG)
            tast = key;
            
        delay = 0;
    }
    // Taste bleibt gedrckt
    else if (pressed)
    {
        if (delay < 0xfe)
            delay++;
    }
    // Taste wurde losgelassen
    else if (release)
    {
        if (mode == KM_LONG && delay != 0xff)
            tast = key;
    }

    // Langer Tastendruck?
    if (mode == KM_LONG)
    {
        if (delay == KEYDELAY_LONG)
        {
            tast = KEY_LONG + key;
            delay = 0xff;
        }
        ptast->delay = delay;
    }
    // Auto-Repeate
    else if (mode == KM_REPEAT)
    {
        if (delay == KEYREPEAT_DELAY)
        {
            tast = key;
            delay = KEYREPEAT_DELAY - KEYREPEAT;
        }
        ptast->delay = delay;
    }

    taster = tast;
}
